/* Copyright (C) 2005, 2006 Hartmut Holzgraefe, MySQL AB

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#include "config.h"

#include <stdio.h>
#include <stdlib.h>
#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif
#ifdef HAVE_GETOPT_H
#include <getopt.h>
#endif

#include <mysql.h>
#include <NdbApi.hpp>

Ndb_cluster_connection *example_init(int *pargc, char ***pargv)
{
	char *connectstr = "localhost";
	int no_retries    = 4;
	int retry_delay   = 5; 
	int verbose       = 1;
	int timeout_first = 30;
	int timeout_rest=10;
	int stat;

	Ndb_cluster_connection *conn;

	// initialize MySQL and Ndb client libraries
	if (ndb_init()) {
		exit(EXIT_FAILURE);
	}

#if 0
	// this worked fine in 5.0 but breaks in 5.1 ..
	// import configuration 
	const char * mycnf_sections[] = {"mysql_cluster", NULL};
	load_defaults("my", mycnf_sections, pargc, pargv);
#endif

	// parse command line arguments
	char *short_options = "c:t:";
	int opt;

#ifdef HAVE_GETOPT_LONG
	struct option long_options[] = {
		{"connect_string", 1, 0, 'c'},
		{"try_reconnect" , 1, 0, 't'},
		{0, 0, 0, 0}
	};
	int option_index;
#endif

	opterr = 0;
	do {
#ifdef HAVE_GETOPT_LONG
		opt = getopt_long(*pargc, *pargv, short_options, long_options, &option_index);
#else
		opt = getopt(*pargc, *pargv, short_options);
#endif 
		switch (opt) {
		case -1: 
		  break;
		case 'c':
			connectstr = optarg;
			break;
		case 't':
			no_retries = atoi(optarg);
		default:
		    printf("invalid command line options\n");
		    exit(EXIT_FAILURE);
			break;
		}
	} while ( opt != -1);

	// prepare connection to cluster
	printf("trying to connect to '%s'\n", connectstr);
	conn = new Ndb_cluster_connection(connectstr);

	// initiate connection
	if (conn->connect(no_retries, retry_delay, verbose))
	{
		fprintf(stderr, "Unable to connect to cluster.");
		exit(EXIT_FAILURE);
	}
	
	// wait for the storage nodes (ndbd's)
    if ((stat = conn->wait_until_ready(timeout_first, timeout_rest)) != 0)
	{
	  if (stat < 0) {
		fprintf(stderr, "Cluster was not ready within %d secs.\n", timeout_first);
		exit(EXIT_FAILURE);
	  } else {
		fprintf(stderr, "Warning: some cluster nodes seem to be down (waited %d secs.)\n", timeout_first + timeout_rest);
	  }
	}

	return conn; 
}

void example_end(Ndb_cluster_connection *conn)
{
	// tear down connection
	delete conn;

	// shut down MySQL and Ndb client libraries
	ndb_end(0);
}



int main(int argc, char **argv)
{
    Ndb_cluster_connection *conn;
	char constr[1024];

	conn = example_init(&argc, &argv);

	printf("Connect string %s\n", conn->get_connectstring(constr, sizeof(constr)));

	printf("Connect port %d\n", conn->get_connected_port());

	printf("Connected host %s\n", conn->get_connected_host());

	printf("# nodes %d\n", conn->no_db_nodes());

	printf("node id %d\n", conn->node_id());

	example_end(conn);

	return EXIT_SUCCESS;
}
